@@ -335,19 +335,18 @@ JNIEXPORT void JNICALL Java_com_sandklef_searduino_Searduino_haltArduinoCode
 {
   int ret;
 
-  printf ("cancel thread\n");
-
-  seasim_set_halted();
-
   if (arduino_thread[thread_index]!=0)
     {
-      printf ("cancel thread %u\n", (unsigned int)arduino_thread[thread_index]);
+      seasim_set_halted();
+      usleep(200);
+
       ret = pthread_cancel(arduino_thread[thread_index]);
+      usleep(200);
+
       arduino_thread[thread_index]=0;
-      printf ("cancel thread %d\n", ret);
     }
 
-  printf ("cancel thread returning\n");
+  usleep(100);
   return ;
 }
 
@@ -398,14 +397,13 @@ JNIEXPORT void JNICALL Java_com_sandklef_searduino_Searduino_startArduinoCode
 
   if (arduino_thread[thread_index]!=0) 
     {
-      printf ("cancel thread %u\n", (unsigned int)arduino_thread[thread_index]);
+      //      pthread_join(arduino_thread[thread_index], (void**)&retval);
+      //printf ("starting thread....join returned: %d\n", retval);
+
       ret = pthread_cancel(arduino_thread[thread_index]);
-      printf ("cancel thread %d\n", ret);
       arduino_thread[thread_index]=0;
     }
 
-  //pthread_join(arduino_thread[thread_index], (void**)&retval);
-  //printf ("starting thread....join returned: %d\n", retval);
 
   //  thread_index++;
 
@@ -415,10 +413,8 @@ JNIEXPORT void JNICALL Java_com_sandklef_searduino_Searduino_startArduinoCode
     printf ("starting thread....thread: %p\n", &arduino_thread);
   */
 
-  printf ("Call seasim_set_running()\n");
   seasim_set_running();
 
-  printf ("Call pthreade_create %d %d \n", thread_index, arduino_code_impl);
   pthread_create(&arduino_thread[thread_index], NULL, arduino_code_impl, NULL);
 }
 
@@ -478,7 +474,15 @@ JNIEXPORT jint JNICALL Java_com_sandklef_searduino_Searduino_setBoardName
 
   ret = seasim_set_board_name((char*)strCIn);
   
-  
+  return ret;
+}
+
+
+JNIEXPORT jint JNICALL Java_com_sandklef_searduino_Searduino_setUpBoard
+  (JNIEnv *env, jobject obj)
+{
+  jint ret = seasim_setup_board();
+  return ret;
 }
 
 
@@ -498,10 +502,7 @@ JNIEXPORT jint JNICALL Java_com_sandklef_searduino_Searduino_hasGenericPinType
 {
   jint ret =  seasim_has_generic_pin_type(pin, type);
 
-  //  printf ("SEASIM RET: %d\n", ret);
-
   return ret;
-
 }
 
 JNIEXPORT jint JNICALL Java_com_sandklef_searduino_Searduino_setGenericInput
@@ -585,3 +586,75 @@ JNIEXPORT jint JNICALL Java_com_sandklef_searduino_Searduino_fakeDigitalInput
 {
   return seasim_fake_digital_input (pin, val);
 }
+
+JNIEXPORT jint JNICALL Java_com_sandklef_searduino_Searduino_getBoardPins
+  (JNIEnv *env, jobject obj, jstring board)
+{
+  JNIEnv * g_env;
+  int ret;
+
+  int getEnvStat = g_vm->GetEnv((void **)&g_env, JNI_VERSION_1_4);
+  CHECK_JNI(getEnvStat, g_env, g_vm);
+
+  const char* strCIn = (env)->GetStringUTFChars(board , 0);
+  //  printf ("board: %s\n", strCIn);
+
+  ret = seasim_get_board_pins((char*)strCIn);
+  
+  return ret;
+
+}
+
+JNIEXPORT jstring JNICALL Java_com_sandklef_searduino_Searduino_getSystemInformation
+  (JNIEnv *, jobject)
+{
+  #define SYS_INFO_LENGTH 2048
+  char buf[SYS_INFO_LENGTH];
+  const char *ver   = seasim_get_searduino_version();
+  const char *board = seasim_get_board_name();
+  const char *codeN = seasim_get_arduino_code_name();
+
+  strcpy(buf, "Searduino system information:\n");
+  strcat(buf, "-----------------------------");
+
+#define ADD_TEXT_IF_OK(a)   if ( (a!=NULL) && ( (strlen(buf)+strlen(a)+2) <SYS_INFO_LENGTH)) \
+    {  strcat(buf, a);   }
+  
+  ADD_TEXT_IF_OK("\n * Version: ");
+  ADD_TEXT_IF_OK(ver);
+  ADD_TEXT_IF_OK("\n * Board:   ");
+  ADD_TEXT_IF_OK(board);
+  ADD_TEXT_IF_OK("\n * Code:    ");
+  ADD_TEXT_IF_OK(codeN);
+
+  JNIEnv * g_env;
+  int getEnvStat = g_vm->GetEnv((void **)&g_env, JNI_VERSION_1_4);
+  jstring jstrBuf = (g_env)->NewStringUTF(buf);
+
+  return jstrBuf;
+}
+
+JNIEXPORT jstring JNICALL Java_com_sandklef_searduino_Searduino_getSupportedBoards
+  (JNIEnv *, jobject)
+{
+  const char *src = seasim_get_supported_boards();
+
+  JNIEnv * g_env;
+  int getEnvStat = g_vm->GetEnv((void **)&g_env, JNI_VERSION_1_4);
+  jstring jstrBuf = (g_env)->NewStringUTF( src);
+
+  return jstrBuf;
+}
+
+JNIEXPORT jstring JNICALL Java_com_sandklef_searduino_Searduino_getBoardSetup
+  (JNIEnv *, jobject)
+{
+  const char *src = seasim_get_board_setup();
+
+  JNIEnv * g_env;
+  int getEnvStat = g_vm->GetEnv((void **)&g_env, JNI_VERSION_1_4);
+  jstring jstrBuf = (g_env)->NewStringUTF( src);
+
+  return jstrBuf;
+}
+
