@@ -34,6 +34,14 @@ import javax.swing.JScrollPane;
 import javax.swing.JLabel;
 import javax.swing.JTabbedPane;
 
+import javax.swing.JOptionPane;
+
+import java.util.Arrays;
+import javax.swing.border.Border;
+import javax.swing.border.LineBorder;
+
+import javax.swing.*;
+
 import com.sandklef.searduino.Searduino;
 import com.sandklef.searduino.SearduinoObserver;
 
@@ -47,28 +55,26 @@ import java.util.Date;
 
 public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, PinEvent, BoardEvent, FileEvent, DirEvent, InoDirEvent, ArduinoCodeNameEvent, JearduinoEvent {
 
-    Searduino searduino ;
+    Searduino            searduino ;
     
     JearduinoPreferences jpref;
     JearduinoMenu        jmenu;
-    GridBagConstraints constraints;
+    GridBagConstraints   constraints;
 
-    PinTable pins;
-    Logger serial ;
-    Logger logger ;
-    FileLogger fileLogger ;
-    LCD    lcd ;
-    InfoPanel infoPanel;
-    ProjectPanel projectPanel;
+    PinTable      pins;
+    Logger        serial ;
+    Logger        logger ;
+    FileLogger    fileLogger ;
+    LCD           lcd ;
+    InfoPanel     infoPanel;
+    ProjectPanel  projectPanel;
 
-    Container topPane; 
-    JPanel    pane;
-    JTabbedPane tabbedPane;
+    Container     topPane; 
+    JPanel        pane;
+    JTabbedPane   tabbedPane;
 
-    private int codeNamesToStore ;
-    private int nrpins = 0;
+    private int   codeNamesToStore ;
     public String version;
-    //    public String boardName;
 
     JPanel topPanel;
     JPanel controlPanel;
@@ -77,10 +83,17 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
     JPanel loggerPanel;
     JPanel fileLoggerPanel;
 
+    /*
+    JPanel commandPanel;
+
+    JLabel lastCommand;
+    JLabel lastCommandResult;
+    */
+
     ExecControl ec;
     Board       board;
     
-    String currentSearduinoProject = "";
+    JearduinoState jState;
 
     static int appSizeHeight = 700;
     static int appSizeWidth  = 1080;
@@ -90,6 +103,8 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 
     public Jearduino() {
 
+	jState = new JearduinoState();
+    
 	jpref = new JearduinoPreferences();
 
 	codeNamesToStore = jpref.getArduinoCodeNameCount();
@@ -117,8 +132,10 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 
 	loggerPanel = new JPanel();
 	fileLoggerPanel = new JPanel();
-
-
+	/*
+	commandPanel = new JPanel();
+	commandPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
+	*/
 	pinPanel.setVisible(true);
 	pinPanel.setLayout(new GridLayout());
 
@@ -133,39 +150,53 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	jmenu = new JearduinoMenu(this, this, this, this, this, this);
 	setJMenuBar(jmenu);
 
-	board = new Board(this);
 	
 	//	controlPanel.add(board);
 
 	searduino = new Searduino();
+	board = new Board(this, searduino);
 	infoPanel = new InfoPanel();
 	projectPanel = new ProjectPanel();
 
 	logger    = new Logger( "Log");
-	fileLogger = new FileLogger( "File Log");
+	fileLogger = new FileLogger( "File Log", this);
 	serial = new Logger( "Serial");
 	lcd    = new LCD( "LCD");
 
 	serial.setMaximumSize(new Dimension(200, 400)); 
 	logger.setMaximumSize(new Dimension(200, 400)); 
-	fileLogger.setMaximumSize(new Dimension(200, 1000)); 
+	//fileLogger.setMaximumSize(new Dimension(600, 800)); 
 	lcd.setMaximumSize(new Dimension(200, 400)); 
 
 	serial.setPreferredSize(new Dimension(200, 100)); 
 	logger.setPreferredSize(new Dimension(600, 150)); 
-	fileLogger.setPreferredSize(new Dimension(600, 400)); 
+	//fileLogger.setPreferredSize(new Dimension(600, 800)); 
 	lcd.setPreferredSize(new Dimension(200, 100)); 
 
+	/*
+	lastCommand       = new JLabel("cmd");
+	lastCommandResult = new JLabel("res");
+	lastCommandResult.setVisible(true);
+	lastCommand.setVisible(true);
+	*/
 	logPanel.add(serial);
 	logPanel.add(lcd);
+	/*
+	logPanel.add(commandPanel);
+	commandPanel.add(lastCommand);
+	commandPanel.add(lastCommandResult);
+	commandPanel.setVisible(true);
+	commandPanel.setBorder(BorderFactory.createTitledBorder("Command"));
+	*/
 
 	loggerPanel.add(logger);
 	fileLoggerPanel.add(fileLogger);
 
-	pins = new PinTable(this, searduino);
+	pins = new PinTable(this, jState, searduino);
 	pins.setVisible(true);
+	JScrollPane sp = new JScrollPane(pins);
 	//	pinPanel.removeAll();
-	pinPanel.add(pins);
+	pinPanel.add(sp);
 
         setTitle("Jearduino - Searduino's Java frontend for Arduino simulation");
 
@@ -197,11 +228,15 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
     }
 
     public void addLog(String data) {
+
 	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-M-dd hh:mm:ss");
 	String date = sdf.format(new Date());
 	String text =  "[" + date + " jearduino-logger]: " + data + "\n";
+
+	/*
 	System.out.println("Logger:   " + data);
 	System.out.flush();
+	*/
 
 	logger.addLog(text);
 	fileLogger.addLog(text);
@@ -255,7 +290,7 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	constraints.gridy = 0;
 
 	constraints.gridwidth = 2;
-	System.out.println("projectPanel" + projectPanel + " c " + constraints);
+	//	System.out.println("projectPanel" + projectPanel + " c " + constraints);
 	pane.add(projectPanel, constraints);
 
 
@@ -323,33 +358,60 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
     }
 
 
+    public boolean isBoardSupported(String boardS) {
+	return board.isSupported(boardS);
+    }
+
+    public String[] getSupportedBoards() {
+	return board.getSupported();
+    }
+
     private void setupBoardPins()
     {
-	logger.addLog("setupBoardPins() <---------------");
+	int ret=0;
+	String bName = jState.getBoard();
 
-	String bName  = searduino.getBoardName();
 	if (bName==null) {
 	    return ;
 	}
 	
-	jpref.setBoard(bName);
-	infoPanel.setBoardName(bName);
-	
-	nrpins = searduino.getNrOfPins();
+	jState.setBoardPins(searduino.getBoardPins(bName));
 
-	version = searduino.getSearduinoVersion();
+	if ( jState.getBoardPins() <= 0) {
+	    System.out.println("setupBoardPins() --------------- too few pins, leaving " + jState.getBoardPins());
+	    return;
+	}
 
 	pins.setupPins();
+
+	ret = searduino.setUpBoard();
+	
+	pins.setupPinTypes();
+
+	jpref.setBoard(bName);
+	    
+	infoPanel.setBoardName(bName);
+	
+	version = searduino.getSearduinoVersion();
     }
 	
     
 
-    private void setupBoard(String bName)
+    private int setupBoard(String bName)
     {
-	String ver = searduino.getSearduinoVersion();
-	searduino.setBoardName(bName);
+	int ret;
 
+	ret = searduino.setBoardName(bName);
+
+	if (ret==0) {
+	    /*	    ec.unsetAll();*/
+	    System.out.println("Failed setting board name....");
+	    return 1;
+	}
+
+	jState.setBoard(bName);
 	setupBoardPins();
+	return 0;
     }
 
 
@@ -376,7 +438,7 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 
     public void handleBoardChoiceEvent(String bName)
     {
-	//	System.out.println("EVENT: Board choice -------------> BOARD: " + bName );
+	stopArduinoCode(false);
 	setupBoard(bName);
     }
 
@@ -384,7 +446,7 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	if (pin==0) return;
 
 	// HESA HESA HESA
-	addLog("EVENT: PIN MODE JAVA (really) pin[" + pin + "]: mode=" + mode );
+	//	addLog("EVENT: PIN MODE JAVA (really) pin[" + pin + "]: mode=" + mode );
 
 	// System.out.println("JAVA (really) pin[" + pin + "]: mode=" + mode );
 	pins.setMode(pin,mode);
@@ -425,11 +487,17 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	else if (type==ExecControl.EXEC_CONTROL_START)
 	    {
 		addLog("START");
+		stopArduinoCode(true);
+		boolean hasDir=false;
+		if (!jState.getCurrentSearduinoProject().equals("")) {
+		    hasDir=true;
+		}
+		useCode(new File(jState.getCanonicalCodeName()), hasDir);
 		searduino.startArduinoCode();
 	    }
     }
 
-    public void getAndUseArduinoCodeName(String code)
+    public void getAndUseArduinoCodeName(String code, boolean hasProject)
     {
 	int ret;
 	String codeName;
@@ -438,10 +506,14 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	//	waitAWhile(500);
 
 	/* If no code, use latest */
-	if (code==null) {
-	    codeName = jpref.getArduinoCodeName(1);
-	} else {
+	if (hasProject) {
 	    codeName = code;
+	} else {
+	    if (code==null) {
+		codeName = jpref.getArduinoCodeName(1);
+	    } else {
+		codeName = code;
+	    }
 	}
 	
 	/* Open file */
@@ -450,9 +522,9 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	    addLog("Couldn't open Arduino file: " + code);
 	    return ;
 	}
-	
+
 	/* Load code */
-	openArduinoFileEvent(f);
+	openArduinoFileEvent(f, hasProject);
     }
     
     
@@ -534,7 +606,8 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	/* Move all items down one step */
 	for (int i=(codeNamesToStore-1);i>-1;i--)
 	    {
-		/*		System.out.println("UPDATE MENU["+(i+1)+"]  " + 
+		/*		
+				System.out.println("UPDATE MENU["+(i+1)+"]  " + 
 				jpref.getArduinoCodeName(i) + 
 				" prev: [ " + i + "]" + 
 				jpref.getArduinoCodeName(i+1));  
@@ -543,92 +616,135 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 					 jpref.getArduinoCodeName(i));
 		
 	    }
+
+	System.out.println("\n\n======= STORE " + canonName + "==================== \n+n");
+
 	/* Store the new one in the first spot */
 	jpref.setArduinoCodeName(0, canonName);
 	
-	/* DIsplay code name in project panel */
+	/* Display code name in project panel */
 	projectPanel.setArduinoCodeName(shortName);
 
 	/* Update menu items */
 	showArduinoCodeNameMenu();
     }
 
-    public void setSearduinoProjectInfo(String s) {
-	if (s==null) { 
-	    s=""; 
+    public void showError(String err) {
+	addLog(err);
+	    JOptionPane.showMessageDialog(this,
+					  "Failed to " + err ,
+					  err + " error",
+					  JOptionPane.ERROR_MESSAGE);
+    }
+
+
+    public void useCode(File f, boolean hasDirectory)  {
+	int ret = 1;
+
+	String code=null;
+	String canonCode=null;
+
+	addLog("useCode(" + f + ", " + hasDirectory + ")");
+
+	try {
+	    code = f.getName();
+	    canonCode = f.getCanonicalPath();
+	    ret = searduino.setArduinoCodeName(canonCode);
 	}
-	if ( s.equals("") ) { 
+	catch (java.lang.UnsatisfiedLinkError e) {
+	    showError("load Arduino code: " + f);
+	    return ;
+	}
+	catch (IOException e)  {
+	    showError("load Arduino code: " + f);
+	    return;
+	}
+
+
+	if (ret != 0 ) {
+	    if (jState.buildTypeStub()) {
+		setSearduinoProjectInfo(jState.getCurrentSearduinoProject(),
+					"", "");
+		showError("load Arduino code: " + f);
+	    }
+	    projectPanel.setArduinoCodeName("");
+	    ec.unsetAll();
+	    return;
+	} else {
+	    addLog("Loaded Arduino code: " + f);
+	    
+	    /* Set up pins */
+	    setupBoardPins();
+
+	    if (f.isDirectory()) {
+		setSearduinoProjectInfo(code, "", "");
+		ec.setStartable();
+	    } else {
+		String dir;
+		if (hasDirectory) {
+		    dir = jState.getCurrentSearduinoProject();
+		} else {
+		    dir = "";
+		} 
+		setSearduinoProjectInfo(dir, code, canonCode);
+		saveArduinoCodeName(canonCode, code);
+		ec.setStartable();
+	    }
+	}
+    }
+
+    public void setSearduinoProjectInfo(String project, String shortCode, String longCode) {
+
+	/* Manage project info */
+	if (project==null) { 
+	    project=""; 
+	}
+	if (project.equals("")) { 
 	    searduino.closeArduinoCode();
 	}
 	
 	/* Set current project name */
-	currentSearduinoProject = s;
-	projectPanel.setProjectName(s);
+	jState.setCurrentSearduinoProject(project);
+	projectPanel.setProjectName(project);
+
+	/* Manage code info */
+	if (shortCode!=null) {
+	    jState.setCodeName(shortCode, longCode);
+	    projectPanel.setArduinoCodeName(shortCode);
+	}
     }
     
     public void handleArduinoFileEvent(File f) {
-	
-	stopArduinoCode(true);
-	//	waitAWhile(500);
-
-	/* Set up pins */
-	setupBoardPins();
-
-	openArduinoFileEvent(f);
-	setArduinoFileInfo("");
-	setSearduinoProjectInfo("");
+	openArduinoFileEvent(f, false);
     }
 
-    public void setArduinoFileInfo(String s) {
-	projectPanel.setArduinoCodeName(s);
-	projectPanel.setProjectName("");
-	setSearduinoProjectInfo(s);
-    }
-
-    public void openArduinoFileEvent(File f)
+    public void openArduinoFileEvent(File f, boolean hasProject)
     {
-	String boardCode = "";
 	int ret = 0;
+	String canonFile=null;
 
 	addLog("Opening file: " + f);
 
-	setArduinoFileInfo("");
+	//	System.out.println("openArduinoFileEvent -> setupBoardPins()");
+	//setupBoardPins();
 
 	try {
-	    boardCode = f.getCanonicalPath();
-	    ret = searduino.setArduinoCodeName(boardCode);
-	}
-	catch (java.lang.UnsatisfiedLinkError e) {
-	    projectPanel.setArduinoCodeName("");
-	    addLog("Failed loading Arduino code: " + f);
-	    return ;
-	}
-	catch (java.io.IOException e)  {
-	    System.out.println("Uh oh... could not get file name" );
-	    addLog("Failed loading Arduino code: " + f.toString());
+	    canonFile = f.getCanonicalPath();
+	} catch (IOException e)  {
+	    showError("load Arduino code: " + f.toString());
+	    return;
 	}
+
+
 	
-	if (ret != 0 ) {
-	    setArduinoFileInfo("");
-	    addLog("Failed loading Arduino code: " + f);
-	    ec.unsetAll();
+	if (hasProject) {
+	    setSearduinoProjectInfo(jState.getCurrentSearduinoProject(), 
+				    f.getName(),
+				    canonFile) ;
 	} else {
-	    addLog("Loaded Arduino code: " + f);
-
-	    /* Set up pins */
-	    setupBoardPins();
-
-	    ///	    projectPanel.setProjectName("");
-	    if (f.isDirectory()) {
-		projectPanel.setProjectName(f.toString());
-	    } else if (f.isFile()) {
-		projectPanel.setArduinoCodeName(f.toString());
-	    } else 
-
-	    setArduinoFileInfo(boardCode);
-	    saveArduinoCodeName(boardCode, f.getName());
-	    ec.setStartable();
+	    setSearduinoProjectInfo("", f.getName(),canonFile) ;
 	}
+	useCode(f, hasProject);
     }
 
     public void stopArduinoCode(boolean unsetAC) {
@@ -636,22 +752,19 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	searduino.haltArduinoCode();
 	
 	if (unsetAC) {
-	    addLog("---> unset");
-	    // Unload arduino code 
-	    searduino.closeArduinoCode();
-	    addLog("<--- unset");
+	    if (jState.buildTypeStub()) {
+		// Unload arduino code 
+		searduino.closeArduinoCode();
+	    }
 	}
 
-	addLog("--- stopArduinoCode setup pins");
-	setupBoardPins();
-	
-
-	addLog("--- stopArduinoCode setup pins done");
+	//	System.out.println("stopArduinoCode -> setupBoardPins()");
+	// setupBoardPins();
 	addLog("<--- stopArduinoCode");
     }
 
 
-    public void waitAWhile(int del) {
+    public static void waitAWhile(int del) {
 	try {
 	    Thread.sleep(del);
 	} catch (InterruptedException e) {
@@ -661,39 +774,25 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 
     public void handleArduinoCodeNameEvent(int codeIdx)
     {
-	addLog("--- 1 handleArduinoCodeNameEvent");
-	//waitAWhile(1000) ;
-	addLog("---> handleArduinoCodeNameEvent");
 	stopArduinoCode(true);
-	addLog("---  handleArduinoCodeNameEvent ac stoped");
 
-	//waitAWhile(500) ;
-	
+	setSearduinoProjectInfo("", 
+				jpref.getArduinoCodeName(codeIdx),
+				jpref.getArduinoCodeName(codeIdx));
 
-       	projectPanel.setArduinoCodeName(jpref.getArduinoCodeName(codeIdx));
-
-	//	waitAWhile(500) ;
-
-	addLog("--- 2 handleArduinoCodeNameEvent");
-	projectPanel.setProjectName("");
-	//waitAWhile(500) ;
-	addLog("--- 3 handleArduinoCodeNameEvent");
-	projectPanel.setProjectName(null);
-	projectPanel.setArduinoCodeName(jpref.getArduinoCodeName(codeIdx));
 	searduino.setArduinoCodeName(jpref.getArduinoCodeName(codeIdx));
-	addLog("--- 4 handleArduinoCodeNameEvent");
 	ec.setStartable();
-	addLog("<--- handleArduinoCodeNameEvent");
     }
 
 
-    public void execCommand(String cmd) throws IOException
+    public void execCommand(String cmd, String shortCmd) throws IOException
     {
 	String commandOutput = null ;
 	Process process = null;
 	BufferedReader inStream = null;
 	String line;
-	
+	int ret;
+
 	process = Runtime.getRuntime().exec(cmd);
 	inStream = new BufferedReader(
 				      new InputStreamReader( process.getInputStream() ));  
@@ -704,24 +803,46 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	addLog(commandOutput);
 	System.out.println(" Output from: " + cmd);
 	System.out.println(" 	 " + commandOutput);
+
+	ret = process.exitValue();
+	String resString = "SUCCESS";
+	
+	if (ret!=0) {
+	    resString = "ERROR";
+	}
+
+	/*
+	lastCommand.setText("Last command: " + shortCmd);
+	lastCommandResult.setText(" : " + resString);
+	*/
+
+	addLog(resString + ":");
+	addLog("  Command:      " + cmd );
+	if (ret!=0) {
+	    addLog("  return value: " + ret);
+	    showError(shortCmd);
+	}
     }
 
-    public void craeteSearduinoProjectFromIno()
-    {
-    }
-    
 
     public void handleJearduinoEvent(int i, Object o) {
 	if (i==JearduinoEvent.JEARDUINO_EVENT_BUILD_PROJECT) {
-	    try {
-		if (currentSearduinoProject!=null) {
-		    buildSearduinoProject(new File(currentSearduinoProject));
-		    
-		}
-	    }
-	    catch (IOException e) {
-		addLog("Failed opening file: " + currentSearduinoProject);
-	    }
+
+	    jState.setCurrentBuildType(jState.SEARDUINO_STATE_STUB);
+	    buildSearduinoProject(jState.getCurrentSearduinoProject());
+
+	} else if (i==JearduinoEvent.JEARDUINO_EVENT_BUILD_ARDUINO) {
+
+	    jState.setCurrentBuildType(jState.SEARDUINO_STATE_ARDUINO);
+	    buildSearduinoProject(jState.getCurrentSearduinoProject(), 
+				  searduino.getBoardName(), false);
+
+	} else if (i==JearduinoEvent.JEARDUINO_EVENT_UPLOAD) {
+
+	    jState.setCurrentBuildType(jState.SEARDUINO_STATE_ARDUINO);
+	    buildSearduinoProject(jState.getCurrentSearduinoProject(), 
+				  searduino.getBoardName(), true);
+
 	}
     }
 
@@ -729,13 +850,27 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 
     private void getAndUseSearduinoDir(File dir)
     {
-	String searduinoDir = System.getProperty("searduino.project.dir") ;
-	String shortDir     = dir.getName();
-	String libName      = searduinoDir + "/" + shortDir  + "/lib" + shortDir.toLowerCase() + ".so";
-
 	stopArduinoCode(true);
-	
-	getAndUseArduinoCodeName(libName);
+	String libName  = null;
+	String shortDir = dir.getName();
+
+	try {
+	    if(dir.exists()) { 
+		libName = dir.getCanonicalPath() + "/lib" + shortDir.toLowerCase() + ".so";
+		//		System.out.println(" --------------- : getAndUseArduinoCodeName(" + libName +");  had:: " + dir);
+	    } else {
+		String searduinoDir = System.getProperty("searduino.project.dir") ;
+		
+		libName      = searduinoDir + "/" + shortDir  + "/lib" + shortDir.toLowerCase() + ".so";
+		//System.out.println(" --------------- : getAndUseArduinoCodeName(" + libName +");  had:: " + dir);
+	    }
+	} catch (IOException e) {
+	    addLog("Failed opening directory: " + dir);
+	    addLog("  Exception: " + e);
+	    return;
+	}
+	setSearduinoProjectInfo(dir.toString(), "", "");
+	getAndUseArduinoCodeName(libName, true);
     }
 
     public void handleSearduinoDirEvent(File dir)
@@ -743,21 +878,91 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	stopArduinoCode(true);
 
 	getAndUseSearduinoDir(dir);
-	setSearduinoProjectInfo(dir.toString());
     }	
 
 
-    private void buildSearduinoProject(File  dir) throws IOException
+    private void buildSearduinoProject(File  dir, String board, boolean upload) 
     {
-	String shortDir     = dir.getName();
-	String searduinoDir = System.getProperty("searduino.project.dir") ;
-	String buildCommand = System.getProperty("searduino.dir") + "/bin/searduino-builder " ;
-	String buildArgs    = searduinoDir + "/" + shortDir;
+	/*
+	  String shortDir     = dir.getName();
+	  String searduinoDir = System.getProperty("searduino.project.dir") ;
+	*/
+	String shortCmd="build";
+	String buildCommand = System.getProperty("searduino.dir") + 
+	    "/bin/searduino-builder " ;
+	
+	String boardArgs="";
+	if (board.equals("")) {
+	    board = "stub";
+	}
 
-	buildCommand = buildCommand + " " + buildArgs;
+	String uploadArgs="";
+	if (upload) {
+	    uploadArgs=" --upload ";
+	    shortCmd="Upload";
+	}
+	
+	boardArgs = " --board " + board + " ";
+	    //searduinoDir + "/" + shortDir ;
+	
+	try { 
+	    String libName  = null;
+	    String shortDir = dir.getName();
+	    if(dir.exists()) { 
+		libName = dir.getCanonicalPath() ;
+		//		System.out.println(" --------------- : getAndUseArduinoCodeName(" + libName +");  had:: " + dir);
+	    } else {
+		String searduinoDir = System.getProperty("searduino.project.dir") ;
+		
+		libName      = searduinoDir + "/" + shortDir  ;
+		//		System.out.println(" --------------- : getAndUseArduinoCodeName(" + libName +");  had:: " + dir);
+	    }
+	    
+	    
+	    String buildArgs    = boardArgs + uploadArgs + " " + libName;
+	    buildCommand = buildCommand + " " + buildArgs;
+	    //	    System.out.println("BUILD: " + buildCommand);
+	    execCommand(buildCommand, shortCmd + " (" + libName + ") to " + board);
 
-	execCommand(buildCommand);
-	getAndUseSearduinoDir(dir);
+	    if (jState.buildTypeStub()) {
+		getAndUseSearduinoDir(dir);
+	    } else {
+		ec.unsetAll();
+	    }
+	} catch (IOException e) {
+	    addLog("Failed building file: " + jState.getCurrentSearduinoProject());
+	}
+    }
+    
+    private void buildSearduinoProject(File  dir, String board) 
+    {
+	buildSearduinoProject(dir, board, false) ;
+    }
+    
+    private void buildSearduinoProject(String  dirS, String board) 
+    {
+	if (dirS==null) return ;
+
+	File dir = new File(dirS);
+	buildSearduinoProject(dir, board);
+    }
+
+    private void buildSearduinoProject(String  dirS, String board, boolean upload) 
+    {
+	if (dirS==null) return ;
+
+	File dir = new File(dirS);
+	buildSearduinoProject(dir, board, upload);
+    }
+
+    private void buildSearduinoProject(String dirS) 
+    {
+	buildSearduinoProject(dirS, "stub");
+    }
+
+    private void buildSearduinoProject(File dir) 
+    {
+	buildSearduinoProject(dir, "stub");
     }
 
     private void createSearduinoFromIno(File dir) throws IOException
@@ -768,17 +973,24 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 
 	createCommand = createCommand + " " + createArgs;
 
-	execCommand(createCommand);
+	execCommand(createCommand, "Import INO dir (" + dir + ")");
     }
 
-    
+    public boolean validBoard() {
+	return 	(!searduino.getBoardName().equals("<none>"));
+    }
 
     public void handleInoDirEvent(File dir) {
 	try {
+	    /* Create Searduino project from ino file */
 	    createSearduinoFromIno(dir);
-	    buildSearduinoProject(dir);
-	    getAndUseSearduinoDir(dir);
-	    setSearduinoProjectInfo(dir.toString());
+
+	    /* Go to Searudino project dir */
+	    String searduinoDir = System.getProperty("searduino.project.dir") ;
+	    String projectDir = dir.getName();
+	    buildSearduinoProject(projectDir);
+	    //	    getAndUseSearduinoDir(new File(projectDir));
+	    //	    setSearduinoProjectInfo(projectDir.toString(), "", "");
 	} catch(IOException e) {
 	    addLog("ERROR when building from INO file");
 	    e.printStackTrace();  
@@ -786,20 +998,31 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	}
     }
 
+    public String getJearduinoState() {
+	return jState.toString();
+    }
+
+    public String getSearduinoState() {
+	return searduino.getSystemInformation();
+    }
+
    public static void main(String[] args) {
 
 	final Jearduino jearduino ;
 	jearduino = new Jearduino();
+
 	boolean startDirect = false;
+	boolean buildDirect = false;
 	int nrArgs     = args.length;
-	String board   = null;
+	String boardS  = "<none>";
 	String code    = null;
 	String project = null;
+	int ret=0;
 
 	for (int i=0; i<nrArgs; i++)  {
 	    if (args[i].equals("--board")) {
 		if (i+1<nrArgs) 			    {
-		    board = args[i+1];
+		    boardS = args[i+1];
 		    i++;
 		} else  {
 		    System.err.println("Missing args to --board");
@@ -807,7 +1030,7 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 		}
 	    }
 	    else if (args[i].equals("--arduino-code")) {
-		if (i+1<nrArgs) 			    {
+		if (i+1<nrArgs) {
 		    code = args[i+1];
 		    i++;
 		} else  {
@@ -820,10 +1043,13 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 		    project = args[i+1];
 		    i++;
 		} else  {
-		    System.err.println("Missing args to --arduino-code");
+		    System.err.println("Missing args to --seardino-project");
 		    System.exit(1);
 		}
 	    }
+	    else if (args[i].equals("--build")) {
+		buildDirect=true;
+	    }
 	    else if (args[i].equals("--start")) {
 		startDirect=true;
 	    }
@@ -840,21 +1066,17 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	    System.exit(1);
 	}
 	
-	if (board==null) {
-		jearduino.setupBoard(jearduino.jpref.getBoard());
-	} else {
-	    jearduino.setupBoard(board);
+
+	if (boardS==null) {
+	    boardS = jearduino.jpref.getBoard();
 	}
 
-	if (project!=null) {
-	    jearduino.handleSearduinoDirEvent(new File(project));
-	} else if (code!=null) {
-	    jearduino.getAndUseArduinoCodeName(code);
-	} else {
-	    /* If no code supplied on cli - use latest */
-	    jearduino.getAndUseArduinoCodeName(null);
+	if (ret!=0) {
+	    System.out.println ("Could not set up board " +  boardS + " returned: " + ret);
+	    System.exit(ret);
 	}
 
+
 	jearduino.fileLogger.start();
 
 	/* Update menu items */
@@ -863,10 +1085,44 @@ public class Jearduino extends JFrame implements SearduinoObserver, ExecEvent, P
 	System.out.println("Searduino version: " + jearduino.version);
 	System.out.println("Searduino board:   " + jearduino.searduino.getBoardName());
 
-	if (startDirect) {
-	    jearduino.ec.sendStart();
+
+	if ( (code==null) && (project==null) ) {
+	    startDirect=false;
 	}
 
+	//	if (jearduino.validBoard()) {
+
+	if ( jearduino.isBoardSupported(boardS)) {
+	    
+	    ret = jearduino.setupBoard(boardS);
+	} else {
+	    System.out.println ("You supplied an unsupported board: " 
+				+ boardS);
+	    String boards[]   = jearduino.getSupportedBoards();
+	    String boardArray = Arrays.toString(boards);
+	    System.out.println ("   Supported boards: " + boardArray);
+	    System.exit(1);
+	}
+	
+	if (project!=null) {
+	    jearduino.handleSearduinoDirEvent(new File(project));
+	} else if (code!=null) {
+	    jearduino.getAndUseArduinoCodeName(code, false);
+	} else {
+	    /* If no code supplied on cli - use latest */
+	    jearduino.getAndUseArduinoCodeName(null, false);
+	}
+	
+
+	    if (buildDirect) {
+		jearduino.handleJearduinoEvent(JearduinoEvent.JEARDUINO_EVENT_BUILD_PROJECT, null);
+	    }
+	    
+	    if (startDirect) {
+		jearduino.ec.sendStart();
+	    }
+	    //}
+
         SwingUtilities.invokeLater(new Runnable() {
 		public void run() {
 		    jearduino.setVisible(true);
