@@ -65,8 +65,12 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 
     JMenuItem loadFromFile;
 
-    /* PRoject menu */
+    /* Arduino menu */
     JMenuItem importArduinoFileItem;
+    JMenuItem verifyArduinoCodeItem;
+    JMenuItem uploadArduinoCodeItem;
+
+
     JMenuItem openSearduinoProjectItem;
     JMenuItem buildItem;
 
@@ -120,8 +124,6 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 
 	String searduino_dir = System.getProperty("searduino.project.dir");
 
-	System.out.println("findArduinoCode -------------" );
-
 
 	if (searduino_dir==null) {
 	    searduino_dir = searduino_default_path;
@@ -213,7 +215,6 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 
     public void showAbout()
     {
-	System.out.println("Show about");
 	JearduinoAbout jabout = new JearduinoAbout();
 
 	jabout.pack();
@@ -223,7 +224,6 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 
     public void showManual()
     {
-	System.out.println("Show manual");
 	try {
 	    Desktop.getDesktop().open(new File("/opt/share/searduino/doc/manual.html"));
 	} catch (java.io.IOException e)  {
@@ -233,7 +233,6 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 
     public void showJearduinoManual()
     {
-	System.out.println("Show Jearduino manual");
 	try {
 	    Desktop.getDesktop().open(new File("/opt/share/searduino/doc/jearduino.html"));
 	} catch (java.io.IOException e)  {
@@ -258,7 +257,7 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 
     public void actionPerformed(ActionEvent e) {
 
-	System.out.println("MENU click " + e.getActionCommand() + " (" + e.getSource() + ")");
+	//	System.out.println("MENU click " + e.getActionCommand() + " (" + e.getSource() + ")");
 	
 	Object o = e.getSource();
 
@@ -290,23 +289,28 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 	    }
 	else if  ( o == importArduinoFileItem )
 	    {
-		System.out.println("importArduinoFileItem");
 		findInoCode();
 	    }
+	else if  ( o == verifyArduinoCodeItem )
+	    {
+		je.handleJearduinoEvent(JearduinoEvent.JEARDUINO_EVENT_BUILD_ARDUINO, null);
+	    }
+	else if  ( o == uploadArduinoCodeItem )
+	    {
+		je.handleJearduinoEvent(JearduinoEvent.JEARDUINO_EVENT_UPLOAD, null);
+	    }
 	else if  ( o == openSearduinoProjectItem )
 	    {
-		System.out.println("openSearduinoProjectItem");
-
 		findSerduinoProject();
 
 	    }
 	else if ( codeIdx != -1 )
 	    {
-		System.out.println("Code at " + codeIdx +
-				   " was choosen " );
+		// System.out.println("Code at " + codeIdx +
+		// 		   " was choosen " );
 		ae.handleArduinoCodeNameEvent(codeIdx);
-		System.out.println("Code at " + codeIdx +
-				   " was choosen ... call has been made" );
+		// System.out.println("Code at " + codeIdx +
+				   // " was choosen ... call has been made" );
 	    }
     }
     
@@ -346,7 +350,7 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
 	leoItem = new JRadioButtonMenuItem("Leonardo");
 	unoItem = new JRadioButtonMenuItem ("Uno");
 	megaItem = new JRadioButtonMenuItem ("Mega");
-	mega2560Item = new JRadioButtonMenuItem("Mega 2560");
+	mega2560Item = new JRadioButtonMenuItem("Mega2560");
 
 	boardMenu.add(leoItem);
 	leoItem.addActionListener(this);
@@ -366,7 +370,7 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
     public void createSearduinoMenu()
     {
 	openSearduinoProjectItem = new JMenuItem("Open Searduino Project");
-	buildItem = new JMenuItem("Build");
+	buildItem = new JMenuItem("Build for simulator");
 
 	openSearduinoProjectItem.addActionListener(this);
 	buildItem.addActionListener(this);
@@ -381,8 +385,17 @@ public class JearduinoMenu extends JMenuBar implements ActionListener {
     public void createArduinoMenu()
     {
 	importArduinoFileItem = new JMenuItem("Import Arduino Sketches");
+	verifyArduinoCodeItem = new JMenuItem("Verify/build code for Arduino board");
+	uploadArduinoCodeItem = new JMenuItem("Upload to Arduino board");
+
+
 	importArduinoFileItem.addActionListener(this);
+	verifyArduinoCodeItem.addActionListener(this);
+	uploadArduinoCodeItem.addActionListener(this);
+
 	arduinoMenu.add(importArduinoFileItem);
+	arduinoMenu.add(verifyArduinoCodeItem);
+	arduinoMenu.add(uploadArduinoCodeItem);
     }
   
 
