@@ -19,11 +19,33 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-export PATH=/opt/bin:${PATH}
-export LD_LIBRARY_PATH=/opt/lib
-export JAVA_LIBRARY_PATH=/opt/lib
+
+read_file()
+{
+    FILE=$1
+    if [ -f ${FILE} ]
+    then
+	. ${FILE}
+    else
+	echo "Can't find file: $FILE"
+	exit 1
+    fi
+}
+
+SEARDUINO_PATH=/tmp/hesa/TMP_INSTALL3
+SEARDUINO_SHARE=${SEARDUINO_PATH}/share/searduino/
+read_file ${SEARDUINO_SHARE}/scripts/settings
+read_file ${SEARDUINO_SHARE}/scripts/functions
+
+
+export PATH=/tmp/hesa/TMP_INSTALL3/bin:${PATH}
+export LD_LIBRARY_PATH=/tmp/hesa/TMP_INSTALL3/lib
+export JAVA_LIBRARY_PATH=/tmp/hesa/TMP_INSTALL3/lib
+
 
 java -cp ${JAVA_LIBRARY_PATH}/Searduino.jar:${JAVA_LIBRARY_PATH}/Jearduino.jar\
+     -Dsearduino.dir=${SEARDUINO_PATH} \
+     -Dsearduino.project.dir=${SEARDUINO_PROJECT_DIR} \
+     -Dsearduino.arduinoex.dir=${ARDUINO_INO_EX} \
      -Djava.library.path=${JAVA_LIBRARY_PATH} \
       com.sandklef.jearduino.Jearduino $*
-
